'       Copyright 1981-1996 by Gerry J. Danen; all rights reserved.

50000 ' $subtitle:  'INPUT.INC: Controlled Keyboard Input Routine'


'       This is the generalized input routine.  Information is passed to the routine by the input variables
'       described below.  information is passed back to the calling program by the output variables
'       described below that.


'       Input variables:
'       ================

'           MININ$               Minimum input value
'           MAXIN$               Maximum input value
'           DEFAULT$             Default for entry
'           HELP$                Help message
'           CTL$                 Function controls as follows:
'
'           "YY,XX,WW,DD,TREIF"
'            -+ -+ -+ -+ +++++
'             |  |  |  | |||||
'             |  |  |  | ||||+----Flush keyboard first
'             |  |  |  | |||+-----Invisible **1
'             |  |  |  | ||+------Escape allowed
'             |  |  |  | |+-------Required entry
'             |  |  |  | +--------Type **2
'             |  |  |  +----------Number of decimals
'             |  |  +-------------Width of input field
'             |  +----------------Input field's starting column
'             +-------------------Input field's row

'       **1   Invisible applies to types S and s only: for passwords...

'       **2   Data types:   S = UPPERCASE STRING
'                           s = Upper/lowercase string
'                           Y = Yes/No
'                           D = Date
'                           T = Time
'                           # = Number, floating decimal
'                           . = Number, fixed decimal
'                           W = Whole dollars
'                           $ = Dollar/cents
'                           P = Path name
'                           F = File name ... WW=02: drive only
'                                             WW=03: ext only
'                                             WW=08: name only
'                                             WW=10: drive + name
'                                             WW=12: name + ext
'                                             WW=14: drive + name + ext

'       <f>

'       Output variables:
'       =================

'           ENTRY$               The characters entered
'           S.VALU!              The value of the entry ... SINGLE PRECISION
'           D.VALU#              The value of the entry ... DOUBLE PRECISION
'           ESCAPE               True or False
'           ESC                  1=ESCAPE was pressed; 0=no ESCAPE
'           MONTH                Month of a date entry
'           DAY                  Day of a date entry
'           YEAR                 Year of a date entry


50100 ' <f>30

input.sub:
50670         col.yy=01: col.xx=04: col.ww=07: col.dd=10: col.t=13: col.r=14: col.e=15: col.i=16 : col.f=17 : ctl.length=17
50680         GOSUB input.sub.check : GOSUB make.help
50690         IF help$<>"" THEN msg$=help$ : GOSUB status.message

input.main.repeat:              ' This is the main loop of the routine.
50720         ok=true : GOSUB input.prt.def
input.2.repeat:                 ' Secondary loop to collect characters for entry$
50740         esc=0 : escape=false : done=false : GOSUB get.char
50750         IF (ch>=32) AND (ch<=126) THEN GOSUB process.char ELSE GOSUB process.control.char
50760         IF NOT (done OR escape) THEN GOTO input.2.repeat
50770         IF escape THEN GOTO input.sub.done
50780         IF ok THEN GOSUB process.entry
50790         IF NOT ok THEN GOTO input.main.repeat

input.sub.done:
50820         GOSUB cursor.off
50830         IF escape THEN alpha$=default$ : GOSUB display.field
50840         default$="" : RETURN

'                                                                       <f>20
input.prt.def:
50880         COLOR scr.bg,scr.fg: alpha$=default$: GOSUB display.field
50890         IF intype$="D" THEN COLOR scr.fg,scr.bg : PRINT "  ";
50900         LOCATE yy,xx: entry$="": RETURN

get.char:
50930         ch$=""
50940         WHILE LEN(ch$)=0 : ch$=INKEY$ : GOSUB input.print.time : WEND
50950         ch=ASC(ch$) : IF LEN(ch$)=2 THEN ch=-ASC(MID$(ch$,2))
50960         IF ch=16 THEN GOSUB input.scr.to.disk : GOTO get.char      'control-p
50970         RETURN

input.print.time:
51000         GOSUB print.time: LOCATE yy,xx+LEN(entry$),1: RETURN


'       Process a valid input character.  If still within field, show the character and add it to entry$.
process.char:
51050         COLOR scr.bg,scr.fg
51060         IF LEN(entry$)>=wide THEN GOSUB input.ill.char : RETURN
51070         IF intype$<>"s" THEN IF ch>=97 AND ch<=122 THEN ch$=CHR$(ch AND 223)   'convert to upper case
51080         IF numeric THEN IF INSTR("-.0123456789",ch$)=0 THEN GOSUB input.ill.char : RETURN
51090         IF LEN(entry$) =0 THEN LOCATE yy,xx : PRINT SPACE$(wide); : LOCATE yy,xx
51100         entry$=entry$+ch$
51110         IF NOT invisible THEN LOCATE yy,xx+LEN(entry$)-1 : PRINT ch$;     'echo char on screen
51120         RETURN


process.control.char:
51160         IF ch=8   OR ch=-75 OR ch=29 OR ch=127 THEN GOSUB process.back.up : RETURN
51170         IF ch=-3  OR ch=3                      THEN GOSUB process.ctrl.c  : RETURN
51180         IF ch=13  THEN GOSUB process.cr     : RETURN
51190         IF ch=27  THEN GOSUB process.escape : RETURN
51200         IF ch=-59 THEN GOSUB input.help     : RETURN
51210         GOSUB input.ill.char : RETURN


'    Process a carriage return.  check for missing required entries substitute default values for null entries.
process.cr:
51260         IF required.item AND entry$="" THEN GOSUB sound.prompt : RETURN   'required item
51270         IF entry$="" THEN entry$=default$                                 'use default
51280         done=true : RETURN


'    Process a back space.  if not already at start of field, then remove last character from the screen
'    and from entry$.

process.back.up:
51350         x=LEN(entry$): IF x=0 THEN GOSUB input.ill.char: RETURN
51360         IF x=1 THEN entry$="" ELSE entry$=LEFT$(entry$,x-1)
51370         IF invisible THEN RETURN
51380         LOCATE yy,xx+LEN(entry$)+1: COLOR scr.bg,scr.fg: PRINT back.space$;" ";back.space$;
51390         IF x=1 THEN GOSUB input.prt.def
51400         RETURN


'    Process an ESCAPE.  If within the entry, restart it.  It at the beginning of it and escape is
'    allowed, set escape condition.

process.escape:
51470         IF entry$<>"" THEN ok=false : done=true : RETURN   'restart entry
51480         IF MID$(ctl$,col.e,1)="E" THEN esc=1 : escape=true : GOSUB input.prt.def ELSE GOSUB input.ill.char
51490         RETURN


'    Control-C or Control-Break pressed.  Ask for confirmation to quit.

process.ctrl.c:
51550         msg$="^C or ^Break pressed.  Type Q to quit; R to resume" : GOSUB status.message : GOSUB flush.keyboard
pcc.repeat:
51570         cc$=INKEY$
51580         IF cc$="Q" THEN cc$="q"
51590         IF cc$="R" THEN cc$="r"
51600         IF NOT (cc$="q" OR cc$="r") THEN GOTO pcc.repeat
51610         eol.fg=scr.fg: eol.bg=scr.bg: GOSUB clear.btm
51620         IF cc$="q" THEN LOCATE 1,1,1: COLOR scr.fg,scr.bg: CLEAR: CLS: PRINT: PRINT "Process aborted by user": PRINT: END
51630         RETURN


'    Process the entry.
'    If errors are found, the error display routine will set OK to FALSE.

process.entry:
51700         IF invisible THEN GOSUB process.password : IF NOT ok THEN RETURN
51710         IF intype$="S" OR intype$="s" THEN RETURN
51720         GOSUB strip.blanks
51730         IF intype$="Y" THEN GOSUB process.yesno    : RETURN
51740         IF intype$="F" THEN GOSUB process.filename : RETURN
51750         IF intype$="P" THEN GOSUB process.pathname : RETURN
51760         IF intype$="D" THEN GOSUB process.date     : RETURN
51770         IF numeric     THEN GOSUB process.numeric  : RETURN
51780         RETURN


'    Process types #, W, and $

process.numeric:
51840         IF entry$="" THEN entry$="0"
51850         period=0 : minus=0
51860         IF (f.dec AND no.of.decimals=0) OR (intype$="W") THEN valid.char$="-0123456789" ELSE valid.char$="-0123456789."
51870         FOR jj=1 TO LEN(entry$)                                           'check numeric char's
51880             IF MID$(entry$,jj,1)="." THEN period=period+1
51890             IF MID$(entry$,jj,1)="-" THEN minus =minus +1
51900             IF INSTR(valid.char$,MID$(entry$,jj,1))=0 THEN jj=999
51910         NEXT jj
51920         IF minus=1 AND LEFT$(entry$,1)<>"-" THEN msg$="minus sign at the left of the number" : GOSUB input.msg : RETURN
51930         IF jj>=999 OR period>1 OR minus>1 THEN msg$="a proper number (e.g. 12345 or -98.76), not "+quo$+entry$+quo$ : GOSUB input.msg : RETURN

51950         IF VAL(minin$)<VAL(maxin$) THEN _
                  IF VAL(entry$)<VAL(minin$) OR VAL(entry$)>VAL(maxin$) THEN msg$="a value in the range of "+minin$+" to "+maxin$ : GOSUB input.msg : RETURN

51990         IF f.dec OR intype$="$" OR intype$="W" THEN GOSUB p.numeric.decimals

52010         IF LEN(entry$)<wide THEN entry$=SPACE$(wide-LEN(entry$))+entry$
52020         s.valu!=VAL(entry$) : d.valu#=VAL(entry$) : RETURN


p.numeric.decimals:

'       Fix entry$ for proper number of decimals.

52090         IF (f.dec AND no.of.decimals<>0) OR (intype$="$") THEN _
                  IF INSTR(entry$,".")=0 THEN entry$=entry$+"."+STRING$(no.of.decimals+1,"0") ELSE entry$=entry$+STRING$(no.of.decimals+1,"0")
52140         x=INSTR(entry$,".") : x=LEN(entry$)-x-no.of.decimals
52150         IF (f.dec AND no.of.decimals<>0) OR (intype$="$") THEN entry$=LEFT$(entry$,LEN(entry$)-x)
52160         x=INSTR(entry$,".")
52170         IF (f.dec AND no.of.decimals=0) OR (intype$="W") THEN IF x<>0 THEN entry$=LEFT$(entry$,x-1)
52180         IF LEN(entry$)>wide THEN entry$=RIGHT$(entry$,wide)
52190         RETURN


'    Process YES/NO entries.  Shift the first character of the entry to upper case and check for "Y" or
'    "N" character.  Any other is error.

process.yesno:
52260         IF entry$<>"Y" AND entry$<>"N" THEN msg$="Y or N" : GOSUB input.msg
52270         RETURN


'    Validate passwords.

process.password:
52330         IF entry$<>sav.def$ THEN msg$="the proper password" : GOSUB input.msg
52340         RETURN


'    Process file name entries.  Only certain characters are valid.

process.filename:
52400         valid.filename=true : IF entry$="" THEN valid.filename=false
52410         valid.char$="ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789-_:."
52420         IF valid.filename THEN GOSUB process.file.loop
52430         IF (wide<>2) AND (wide<>10) AND (wide<>14) THEN IF colon<>0 THEN valid.filename=false
52440         IF (wide<>12) AND (wide<>14) THEN IF period<>0 THEN valid.filename=false
52450         IF valid.filename THEN RETURN
52460         msg$="a valid file name, not "+quo$+entry$+quo$ : GOSUB input.msg : RETURN

process.file.loop:
52490         IF LEFT$(entry$,1)<"A" OR LEFT$(entry$,1)>"Z" THEN valid.filename=false : RETURN
52500         period=0 : colon=0
52510         FOR i=1 TO LEN(entry$)
52520             IF MID$(entry$,i,1)="." THEN period=period+1
52530             IF MID$(entry$,i,1)=":" THEN colon =colon +1
52540             IF INSTR(valid.char$,MID$(entry$,i,1))=0 THEN valid.filename=false : i=LEN(entry$)
52550         NEXT i
52560         IF period>1 OR colon> 1 THEN valid.filename=false
52570         RETURN


'    Process DOS V2.00+ path names.

process.pathname:
52630         valid.filename=true : valid.char$="ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789-_:.\" : GOSUB process.file.loop
52640         IF NOT valid.filename THEN msg$="a valid path name" : GOSUB input.msg
52650         RETURN


'    Process dates.  Check for proper values in day, month, and year.  If no errors, redisplay date as
'    DD-MM-YY.  Fill in the variables DAY, MONTH, and YEAR.

process.date:
52720         IF LEN(entry$)=5 OR LEN(entry$)=7 THEN entry$="0"+entry$          'leading 0 not typed
52730         IF LEN(entry$)=8 THEN entry$ = MID$(entry$,1,2)+MID$(entry$,4,2)+MID$(entry$,7,2)     'forget "-" or "/" separators
52740         GOSUB verify.the.date : IF NOT valid THEN GOSUB bad.date : RETURN
52750         day=VAL(MID$(entry$,1,2)) : month=VAL(MID$(entry$,3,2)) : year=VAL(MID$(entry$,5,2))
52760         s.valu!= year*10000+month*100+day : entry$=LEFT$(entry$,2)+"-"+MID$(entry$,3,2)+"-"+RIGHT$(entry$,2) : RETURN


verify.the.date:
'                               WILL NOT ALLOW DATES BEFORE 1980...
52810         valid=true : hold.date$=DATE$
52820         ON ERROR GOTO illegal.date
52830         DATE$=MID$(entry$,3,2)+"/"+MID$(entry$,1,2)+"/"+MID$(entry$,5,2)
verify.the.date.resume:
52850         ON ERROR GOTO error.routines
52860         DATE$=hold.date$ : RETURN

illegal.date:
52890         valid=false
52900         RESUME verify.the.date.resume

'               Found a bad date -- report the error
bad.date:
52940         msg$="a date in the DDMMYY format, e.g. 261249 (not "+entry$+")" : GOSUB input.msg : RETURN


'               Print error message, set OK to FALSE.
input.msg:
52990         msg$="Please enter "+msg$ : GOSUB error.message : ok=false : RETURN


'    Print help message when F1 has been pressed.

input.help:
53050         GOSUB make.help: GOSUB input.help.beep
53060         IF help$="" THEN msg$="Sorry, no help available..." ELSE msg$=help$
53070         GOSUB status.message: LOCATE yy,xx+LEN(entry$): RETURN

make.help:
53100         IF help$<>"" THEN RETURN                                          'already filled in
53110         IF numeric THEN help$="Enter a number"
53120         IF numeric AND (VAL(minin$)<VAL(maxin$)) THEN help$=help$+" from "+minin$+" to "+maxin$
53130         IF intype$="Y" THEN help$="Enter a Y for Yes, or an N for No"
53140         IF intype$="D" THEN help$="Enter a date in the format DDMMYY, e.g. 261249"
53150         IF intype$="F" THEN help$="Enter a valid file name"
53160         IF intype$="P" THEN help$="Enter a valid path name"
53170         IF invisible   THEN help$="Enter your password"
53180         RETURN

'               Cursor on and off routines
cursor.on:
53220         COLOR scr.bg,scr.fg: LOCATE yy,xx,1: RETURN

cursor.off:
53250         eol.fg=scr.fg: eol.bg=scr.bg: GOSUB clear.btm: LOCATE yy,xx,0 : minin$="": maxin$="": help$="": PRINT SPACE$(wide);
53260         IF invisible THEN RETURN
53270         COLOR scr.fg,scr.bg: alpha$=entry$: GOSUB display.field: RETURN


display.field:
53310         LOCATE yy,xx: IF wide<LEN(alpha$) THEN alpha$=LEFT$(alpha$,wide)
53320         PRINT alpha$;
53330         IF (wide-LEN(alpha$))>0 THEN PRINT STRING$(wide-LEN(alpha$)," ");
53340         RETURN


'               Set up initial condition, evaluate CTL$, and set standard defaults. <f>30
input.sub.check:
'             Complain if CTL$ length not correct
53410         IF LEN(ctl$)<>ctl.length THEN BEEP : PRINT : PRINT "CTL$ MUST BE"; ctl.length; " CHARACTERS LONG: ";ctl$ : STOP

53430         IF MID$(ctl$,col.f,1)="F" THEN GOSUB flush.keyboard

'    Set YY,XX if in CTL$
53460         IF ASC(ctl$)>=48 AND ASC(ctl$)<=57 THEN yy=VAL(MID$(ctl$,col.yy,2)) : xx=VAL(MID$(ctl$,col.xx,2))
53470         GOSUB cursor.on

53490         wide=VAL(MID$(ctl$,col.ww,2))
53500         IF wide<1 THEN wide=1

53520         intype$=MID$(ctl$,col.t,1) : cc$="SsY#.DW$FP"
53530         IF INSTR(cc$,intype$)=0 THEN intype$="s"
53540         IF intype$="Y" THEN wide=1

53560         month=0 : day=0 : year=0

'    Set defaults
53590         IF intype$="Y" AND default$="" THEN default$="N"
53600         IF INSTR(".#$W",intype$)<>0 THEN numeric=true ELSE numeric=false

'    Maximum width for pathname = 63
53630         IF intype$="P" THEN IF wide>63 THEN wide=63

'    Check for valid width on file names
53660         IF intype$="F" THEN IF (wide<>2 AND wide<>3 AND wide<>8 AND wide<>10 AND wide<>12 AND wide<>14) THEN wide=12

53680         IF MID$(ctl$,col.i,1)="I" THEN invisible=true ELSE invisible=false
53690         IF invisible THEN sav.def$=default$ : default$=""

53710         IF MID$(ctl$,col.r,1)="R" THEN required.item=true ELSE required.item=false
53720         IF numeric AND default$<>"" THEN required.item=false
53730         IF (NOT numeric) AND default$<>"" THEN required.item=false
53740         IF MID$(ctl$,col.r,1)="R" AND invisible THEN required.item=true

53760         IF intype$="." THEN intype$="#" : f.dec=true ELSE f.dec=false
53770         no.of.decimals=VAL(MID$(ctl$,col.dd,2))
53780         IF numeric AND wide>16 THEN wide=16
53790         IF numeric AND no.of.decimals>wide-1 THEN no.of.decimals=wide-1

53810         fmt$=""
53820         IF f.dec THEN IF no.of.decimals=0 THEN fmt$=RIGHT$("#######################",wide) ELSE fmt$=RIGHT$("#######################",wide-1-no.of.decimals)+"."+STRING$(no.of.decimals,"#")
53840         IF f.dec THEN IF LEFT$(fmt$,1)="," THEN MID$(fmt$,1,1)="#"
53850         IF intype$="$" THEN no.of.decimals=2 : fmt$="$"+RIGHT$("###################.##",wide-1) : IF MID$(fmt$,2,1)="," THEN MID$(fmt$,2,1)="#"
53860         IF intype$="W" THEN no.of.decimals=0 : fmt$="$"+RIGHT$("#######################",wide-1) : IF MID$(fmt$,2,1)="," THEN MID$(fmt$,2,1)="#"

53880         IF NOT numeric THEN RETURN
53890         GOSUB input.sci.notation : entry$=default$ : GOSUB strip.blanks
53900         IF wide=>LEN(entry$) THEN default$=SPACE$(wide-LEN(entry$))+entry$ ELSE default$=LEFT$(entry$,wide)
53910         entry$=minin$ : GOSUB strip.blanks : minin$=entry$
53920         entry$=maxin$ : GOSUB strip.blanks : maxin$=entry$
53930         IF VAL(minin$)=>VAL(maxin$) THEN GOSUB input.calc.min.max
53940         RETURN


'               Check for single/double precision scientific notation numbers
input.sci.notation:
53990         entry$=default$ : GOSUB strip.blanks : default$=entry$
54000         IF LEN(default$)<5 THEN RETURN
54010         IF MID$(default$,LEN(default$)-3,1) <> "D" AND MID$(default$,LEN(default$)-3,1) <> "E" THEN RETURN
54020         move.decimal = VAL(RIGHT$(default$,2))
54030         IF MID$(default$,LEN(default$)-2,1) =  "-" THEN _
                  move.left=true : move.rite=false : move.decimal = move.decimal * (-1) _
              ELSE move.rite=true : move.left=false

'             Return if no decimal
54090         jj = INSTR(default$,".")
54100         IF jj=0 THEN IF LEN(default$)>wide THEN default$=RIGHT$(default$,wide)
54110         IF jj=0 THEN RETURN

'             Now lineup decimal point with mask
54140         default$ = "00000000000000000" + default$ + "00000000000000000"
54150         jj = INSTR(default$,".") : default$ = MID$(default$,1,jj-1) + MID$(default$,jj+1)
54160         jj = jj + move.decimal : i = wide - no.of.decimals - 1 : entry$ = MID$(default$,jj-i,i) + "."
54170         default$ = entry$ + MID$(default$,jj,no.of.decimals)
54180         IF LEN(default$) > wide THEN default$=LEFT$(default$,wide)
54190         RETURN

input.calc.min.max:
54220         maxin$=""
54230         FOR i=1 TO LEN(fmt$)
54240             IF MID$(fmt$,i,1)="#" THEN maxin$=maxin$+"9"
54250             IF MID$(fmt$,i,1)="." THEN maxin$=maxin$+"."
54260         NEXT i
54270         minin$="-"+MID$(maxin$,2)
54280         RETURN

input.ill.char:
54310         IF sound.sys THEN SOUND 600,1: SOUND 1800,1 ELSE BEEP
54320         RETURN

input.help.beep:
54350         IF sound.sys THEN SOUND 400,1 ELSE BEEP
54360         RETURN

'       $include: 'SCR2DSK.INC'
'++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
